/*********************************************************************
* NAME: ProcessSkippedDataSets
*
* WHAT: Fetchs and displays "SKIPPED DATA SETS" data set from the 
*       Target Service.
*
* SMS-SPECIFIC ROUTINES CALLED: 
*				NWSMPutOneName (SMS UTILITY API)
*				NWSMTSScanDataSetBegin (TSAPI)
*				NWSMTSOpenDataSetForBackup (TSAPI)
*				NWSMTSReadDataSet (TSAPI)
*				NWSMTSCloseDataSet (TSAPI)
*				NWSMTSScanDataSetEnd (TSAPI)
*
**********************************************************************/
void ProcessSkippedDataSets(void)
{
  /* connection is global value in this example */
  /* dataBuffer is global value in this example */
  CCODE                      ccode;
  NWSM_DATA_SET_NAME_LIST   *resourceName = NULL;
  NWSM_DATA_SET_NAME_LIST   *dataSetNameList = NULL;
  NWSM_DATA_SET_NAME         dataSetName;
  UINT32                     sequence;
  UINT32                     mode = NWSM_USE_LOCK_MODE_IF_DW_FAILS;
  UINT32                     bytesRead;
  UINT32                     dataHandle = NULL;
  UINT32                     bytesToRead;
  UINT32                     leftOverBytes = 0L;
  UINT16                     dataSetNameSize;
  BUFFER                    *bufferEnd = NULL;
  BUFFER                    *bufferPtr = NULL;
  BUFFER                    *readPtr = NULL;
  int                        empty = TRUE;
  int                        getNextName;
  char                       errorMessage[256];

  /* display SKIPPED DATA SETS name */
  printf ("\n\n\"SKIPPED DATA SETS\":\n");

  /* setup resource name list */
  if ((ccode = NWSMPutOneName( (void HUGE **)
                          &resourceName, 
                          NWSM_TSA_DEFINED_RESOURCE_TYPE,
                          0L,                              /* selectionType */
                          FALSE,                           /* reverse order */
                          NULL,                          /* first separator */
                          NULL,                         /* second separator */
                          (void HUGE *)
                          "SKIPPED DATA SETS" )) /* SKIPPED DATA SETS special name */
       != 0 )
  {
    PrintError( 
      ccode, 
      "DE: * * *   ERROR  * * * NWSMPutOneName(SKIPPED DATA SETS)", 
      0);
    goto Terminate;
  }

  /* read in SKIPPED DATA SETS data set */
  if ((ccode = NWSMTSScanDataSetBegin( 
                                 connection,
                                 resourceName,
                                 NULL,                      /* scan control */
                                 NULL,                    /* selection list */
                                 &sequence,
                                 NULL,                  /* scan information */
                                 &dataSetNameList ))
       != 0 )
  {
    PrintError( 
      ccode, 
      "DE: * * *   ERROR   * * * NWSMTSScanDataSetBegin(SKIPPED DATA SETS)", 
      0);
    goto Terminate;
  }
  if (((ccode = NWSMTSOpenDataSetForBackup( 
                                          connection,
                                          sequence,
                                          0L,                       /* mode */
                                          &dataHandle ))
       !=0 ) || ( dataHandle == 0L ) )
  {
    PrintError( 
      ccode, 
      "DE: * * *   ERROR   * * * NWSMTSOpenDataSetForBackup(SKIPPED DATA SETS)", 
      0);
    goto Terminate;
  }

  /* SKIPPED DATA SETS data set is a binary data set layed out in
   * NWSM_DATA_SET_NAME_LIST structure format. Leftover pieces may 
   * be in read buffer. Need to manage the buffer to handle this.
   */

  readPtr = dataBuffer; /* 1st time through, start at beginning */
  leftOverBytes = 0L;
  bytesRead = 0L;
  empty = TRUE;
  do
  {
    bytesToRead = BUFFER_SIZE - leftOverBytes;
    if (( ccode = NWSMTSReadDataSet( 
                               connection,
                               dataHandle,
                               bytesToRead,
                               &bytesRead,
                               readPtr ))
          == 0 )
    {
      if ( ! bytesRead )
      {
        break;
      }
      /* lets parse buffer, output name(s) and check boundary conditions
       * of buffer
       */
      bufferPtr = dataBuffer;
      bufferEnd = dataBuffer + bytesRead;
      getNextName = TRUE;
      while ( getNextName )
      {
        /* make sure there's enough space in buffer to contain a whole 
         * data set. If not, shuffle buffer */
        if (( leftOverBytes = bufferEnd - bufferPtr ) < 4 )
        {
          getNextName = FALSE;
        }
        else
        {
          /* data set name size is within first 4 bytes (2nd UINT16):
           * if data set name size == left over bytes, could handle, but
           * an increment off end will occur, so treat the same as 
           * greater-than condition.
           */
          if (( dataSetNameSize = ((NWSM_DATA_SET_NAME_LIST *) bufferPtr )->
                             dataSetNameSize )
                >= leftOverBytes )
            getNextName = FALSE;
        }
        if (! getNextName )
        {
          /* move left over to top of buffer */
          memcpy( dataBuffer, bufferPtr, leftOverBytes );

          /* prepare for next read */
          readPtr = dataBuffer + leftOverBytes;
          break;
        }
        if ((ccode = NWSMGetOneName( bufferPtr, &dataSetName )) == 0 )
        {
          NWSMConvertError( 
                              connection, 
                              dataSetName.selectionType,
                              errorMessage );
          printf ("\t(0x%08lX) %s: %s\n", 
                     dataSetName.selectionType, 
                     errorMessage, 
                     dataSetName.name );
        }
        else
        {
          PrintError( 
            ccode, 
            "DE: * * *   ERROR   * * * NWSMGetOneName(SKIPPED DATA SETS)", 
            0);
          empty = FALSE;
          break;
        }
        bufferPtr += dataSetNameSize + 2;
      } /* end while ( getNextName ) */
      empty = FALSE;
    } /* end read data set OK */
    else
    {
      PrintError( 
        ccode, 
        "DE: NWSMTSReadDataSet(SKIPPED DATA SETS)", 
        0);
      break;
    }
  } while ( 1 );

Terminate:

  if (empty)
  {
    printf( "\t<empty>\n");
  }

  if ( dataHandle )
  {
    if (( ccode = NWSMTSCloseDataSet(
                                connection,
                                &dataHandle ))
          != 0 && ccode != NWSMTS_INVALID_DATA_SET_HANDLE )
    {
      PrintError( 
        ccode, 
        "DE: * * *   ERROR   * * * NWSMTSCloseDataSet(SKIPPED DATA SETS)", 
        0);
    }
  }
  if (( ccode = NWSMTSScanDataSetEnd( 
                                connection,
                                &sequence,
                                NULL, /* scan information */
                                &dataSetNameList ))
        != 0 )
  {
    PrintError( 
      ccode, 
      "DE: * * *   ERROR   * * * NWSMTSScanDataSetEnd(SKIPPED DATA SETS)", 
      0);
  }
  if ( resourceName )
    free( resourceName );
  if ( dataSetNameList )
    free( dataSetNameList );

} /* ProcessSkippedDataSets() */

/****************************************************************************/